<?php

namespace Toloka\PhpApi;

use Dflydev\FigCookies\Cookies;
use Psr\Http\Client\ClientInterface as HttpClientInterface;
use Psr\Log\LoggerInterface;
use Psr\SimpleCache\CacheInterface;
use Toloka\PhpApi\Traits\AuthTrait;
use Toloka\PhpApi\Traits\HttpHelperTrait;
use Toloka\PhpApi\Traits\TopicTrait;

class Client implements ClientInterface {

  use HttpHelperTrait;
  use AuthTrait;
  use TopicTrait;

  const CACHE_KEY_COOKIES = 'cookies';
  const CACHE_KEY_LOGGED_IN = 'is_logged_in';

  /**
   * @var ClientInterface|null
   */
  private static ?ClientInterface $instance;

  /**
   * Hurtom Toloka base url.
   *
   * @var string
   */
  public static string $base_url = 'https://toloka.to';

  /**
   * User agent to use for requests.
   *
   * @var string
   */
  public static string $user_agent = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36';

  /**
   * Http client.
   *
   * @var HttpClientInterface
   */
  protected HttpClientInterface $httpClient;

  /**
   * Cache bin.
   *
   * @var CacheInterface
   */
  protected CacheInterface $cache;

  /**
   * Events logger.
   *
   * @var LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * Toloka client constructor.
   *
   * @param \Psr\Http\Client\ClientInterface $httpClient
   * @param \Psr\SimpleCache\CacheInterface $cache
   * @param \Psr\Log\LoggerInterface $logger
   */
  public function __construct(
    HttpClientInterface $httpClient,
    CacheInterface $cache,
    LoggerInterface $logger
  ) {
    $this->httpClient = $httpClient;
    $this->cache = $cache;
    $this->logger = $logger;

    if ($cache->has(self::CACHE_KEY_COOKIES)) {
      $persistent_cookies = $cache->get(self::CACHE_KEY_COOKIES);
      if ($persistent_cookies instanceof Cookies) {
        $this->cookies = $persistent_cookies;
      }
    }
  }

  public static function getInstance(): ClientInterface {
    return self::$instance;
  }

}
