<?php

namespace Toloka\PhpApi\Traits;

use GuzzleHttp\Psr7\Utils;
use Symfony\Component\DomCrawler\Crawler;
use Toloka\PhpApi\Exception\Auth\AuthException;
use Toloka\PhpApi\Exception\Auth\InvalidAuthCredentials;

trait AuthTrait {

  /**
   * {@inheritDoc}
   */
  public function login(string $login, string $password): void {
    if ($this->isLoggedIn()) {
      return;
    }

    // Flush cached logged in state.
    $this->cache->delete(self::CACHE_KEY_LOGGED_IN);

    $body = http_build_query([
      'username' => $login,
      'password' => $password,
      'autologin' => 'on',
      'ssl' => 'on',
      'redirect' => '',
      'login' => 'Вхід',
    ]);
    $response = $this->makeRequest('POST', '/login.php', [
      'Content-Type' => 'application/x-www-form-urlencoded',
    ], Utils::streamFor($body), FALSE);
    if ($response->getStatusCode() === 302) {
      // Force check if we logged in successfully, to be sure + cache this info.
      if (!$this->isLoggedIn()) {
        throw new AuthException();
      }
      return;
    }
    elseif ($response->getStatusCode() === 200) {
      // In case of returned page - try to find an error and throw exception.
      $crawler = new Crawler($response->getBody()->getContents());
      $text = $crawler->filter('.forumline .row1 span.gen')->text(NULL, TRUE);
      if (str_contains($text, 'Такий псевдонім не існує, або не збігається пароль.')) {
        throw new InvalidAuthCredentials();
      }
    }
    throw new AuthException();
  }

  /**
   * {@inheritDoc}
   */
  public function isLoggedIn(): bool {
    $cached = $this->cache->get(self::CACHE_KEY_LOGGED_IN, NULL);
    if ($cached === NULL) {
      $response = $this->makeRequest('GET', '/');
      $cached = FALSE;
      if ($response->getStatusCode() === 200) {
        $crawler = new Crawler($response->getBody()->getContents());
        $menu_links = $crawler->filter('table.navie6fix a');
        foreach ($menu_links as $menu_link) {
          // If there is a link to profile, then we authorized.
          if ($menu_link->textContent === 'Профіль') {
            $cached = TRUE;
            break;
          }
        }
      }
      // Save logged in state in cache.
      $this->cache->set(self::CACHE_KEY_LOGGED_IN, $cached);
    }
    return $cached;
  }

  /**
   * {@inheritDoc}
   */
  public function logout(): void {
    if (!$this->isLoggedIn()) {
      return;
    }
    // Flush cached logged in state.
    $this->cache->delete(self::CACHE_KEY_LOGGED_IN);
    $session_id = $this->getUserSessionId();
    if (!$session_id) {
      return;
    }
    $response = $this->makeRequest('GET', '/login.php?logout=true&sid=' . $session_id);
    if ($response->getStatusCode() === 302) {
      $this->cache->set(self::CACHE_KEY_LOGGED_IN, FALSE);
    }
  }

  /**
   * Get current session id.
   *
   * @return string|null
   *   String value if session exists, null otherwise.
   */
  private function getUserSessionId(): ?string {
    if ($this->cookies->has('toloka_sid')) {
      return $this->cookies->get('toloka_sid')->getValue();
    }
    return NULL;
  }

}
