<?php

namespace Toloka\PhpApi\Traits;

use Dflydev\FigCookies\Cookie;
use Dflydev\FigCookies\Cookies;
use Dflydev\FigCookies\FigRequestCookies;
use Dflydev\FigCookies\SetCookies;
use GuzzleHttp\Psr7\Request;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Toloka\PhpApi\Client;

trait HttpHelperTrait {

  /**
   * @var Cookies
   */
  protected ?Cookies $cookies = NULL;

  protected function applyRequestCookies(RequestInterface $request): RequestInterface {
    if (!$this->cookies) {
      return $request;
    }
    foreach ($this->cookies->getAll() as $cookie) {
      $request = FigRequestCookies::set($request, $cookie);
    }
    return $request;
  }

  private function saveCookies($data) {
    if ($data instanceof SetCookies) {
      $cookies = [];
      foreach ($data->getAll() as $cookie) {
        $cookies[] = Cookie::create($cookie->getName(), $cookie->getValue());
      }
      $this->cookies = new Cookies($cookies);
    }
    elseif ($data instanceof Cookies) {
      $this->cookies = $data;
    }
    $this->cache->set(self::CACHE_KEY_COOKIES, $this->cookies);
  }

  public function makeRequest(
    string $method,
    $uri,
    array $headers = [],
    $body = null,
    bool $include_cookies = TRUE,
    bool $save_cookies = TRUE
  ): ResponseInterface {
    $headers['User-Agent'] = Client::$user_agent;
    $request = new Request($method, Client::$base_url . $uri, $headers, $body);
    if ($include_cookies) {
      $request = $this->applyRequestCookies($request);
    }
    $response = $this->httpClient->sendRequest($request);
    if ($save_cookies) {
      $cookies = SetCookies::fromResponse($response);
      // Sometimes server don't respond with cookie header.
      if (count($cookies->getAll()) > 0) {
        $this->saveCookies($cookies);
      }
    }
    return $response;
  }

}
